from JWST_IMAGE_MAKER.importing import get_file
from JWST_IMAGE_MAKER.processing import process_file
from JWST_IMAGE_MAKER.plotting import plot_data

# make_image is the function that the user will call
# This function will then call all of the other modules


def make_image(filenames, save_image):
    """_summary_

    Args:
        filename (list of strings): list containing the name(s) of the fits file(s) used to construct an image. If only one .fits file
                                    is given, please store as a 1 element list



        save_image(T/F): Specifies whether the user wants to save the image to their computer
    """
    file_data = get_file(filenames)
    processed_data = process_file(file_data)
    plot_data(processed_data, filenames, save_image)  # this will also save the image
    pass


#Testing the code:
'''
file_name = [
    "jw02739-o002_t001_miri_f770w_i2d.fits",
    "jw02739-o002_t001_miri_f1500w_i2d.fits",
    "jw02739-o002_t001_miri_f1130w_i2d.fits",
]
make_image(file_name, save_image=False)

'''



import matplotlib.pyplot as plt
import numpy as np
import os

def plot_data(processed_data,filename,save_image):
    """_summary_

    Args:
        processed_data (np.array): Processed data, ideally 2D but can also be 3D. Remember that this will NOT be a multi-row/2-column dataset
    """
    #Add some code here later that makes it look pretty and (maybe) customizes the plotting procedure based on what the image contains

        # showing just the first slice
    for i in range(len(processed_data[0,0,:])): 
        img = processed_data[:,:,i]
        vmin,vmax=np.percentile(img.flatten(), [12,99])  #12 and 99 were determined by trial and error
        plt.figure(i,figsize=(5,5),dpi=175)
        plt.imshow(img, vmin=vmin, vmax=vmax, cmap='bone')
        #saving image if desired by user
        if save_image==True:
            name = os.path.splitext(filename[i])[0].lower()
            new_ext='.pdf'
            path='JWST_IMAGE_MAKER/Output Figures/'
            plt.savefig(path+name+new_ext,format='pdf',dpi=1200,bbox_inches='tight')
        plt.show()



    pass




import numpy as np
from astropy.io import fits
from matplotlib import pyplot as plt

def process_file(file_data):
    """_summary_

    Args:_type_
        file_data (np.array): array containing data from JWST

    Returns:
        processed_data(np.array): _description_
    """


    # TODO: auto adjust based on distribution, account for all slices
    #       account for biases

    processed_data=file_data

    return processed_data


import numpy as np
from astropy.io import fits
import sys
import os

#path = "JWST_IMAGE_MAKER/JWST_IMAGE_MAKER/data/"


def get_file(filename):
    """This function imports the file provided by the user and converts it to a np.array.
    If the given file does not have a .fits extension, the code will send an error to the user and will not attempt to open the file.

    Args:
        filename (str): name of the fits file that the user wants an image of.

    Returns:
        array_data (np.array): A 3D array containing all of the data for every single .fits file given
    """

    # Checking the extension and size of the first filename given by the user
    # Note: this code assumes all given .fits files have the same size
    check_extension(filename[0])
    xdim, ydim = check_size(filename[0])

    # Creating array that can store the data for ALL of the .fits files provided by the user
    # This array will be 3D to ensure it can store the x-y photon data for each wavelength slice
    full_dataset = np.zeros(((xdim, ydim, len(filename))))

    # Looping over all files provided by user and saving their data in full_dataset
    for i in range(len(filename)):
        check_extension(filename[i])
        # Importing fits file
        fits_data = fits.open(filename[i])
        # Converting data from HDUList to np.array
        array_data = fits_data[1].data
        full_dataset[:, :, i] = array_data

    return full_dataset


def check_extension(filename):
    """This function checks the extension of a provided data file

    Args:
        filename (str): name of the specified data file

    Returns:
        void: Stops code if the file is not a .fits file, nothing otherwise
    """
    ext = os.path.splitext(filename)[-1].lower()
    # if the user input file has extension .fits, the code will continue with no problem
    if ext == ".fits":
        pass

    # if the user input file has a different extension, the code will output a warning then stop running
    else:
        print("ERROR: Input file must have a .FITS extension.")
        print("Input File extension is:", ext)
        sys.exit()  # tells code to stop running


def check_size(filename):
    """This function checks the dimensions of a .fits file

    Args:
        filename (str): name of input .fits file

    returns:
        xdim, ydim (int) : The x and y dimensions of the given .fits files
    """
    fits_data = fits.open(filename)
    # Converting data from HDUList to np.array
    array_data = fits_data[1].data
    xdim = len(array_data[:, 0])
    ydim = len(array_data[0, :])
    return xdim, ydim



import pathlib
from setuptools import setup, find_packages

HERE = pathlib.Path(__file__).parent

VERSION = '0.1.4'
PACKAGE_NAME = 'JWST_IMAGE_MAKER'
AUTHOR = 'Henry White and Hansen Jiang'
AUTHOR_EMAIL = 'henrywhite@sidratresearch.com, hansen@sidratresearch.com'
URL = 'https://github.com/henrywhite2727/JWST_Image_Maker'

LICENSE = 'Apache License 2.0'
DESCRIPTION = 'Creates images from raw JWST data'
LONG_DESCRIPTION = (HERE / "README.md").read_text()
LONG_DESC_TYPE = "text/markdown"

INSTALL_REQUIRES = [
      'numpy',
      'astropy.io.fits',
      'sys',
      'os',
      'matplotlib.pyplot' #maybe remove .pyplot later

]

setup(name=PACKAGE_NAME,
      version=VERSION,
      description=DESCRIPTION,
      long_description=LONG_DESCRIPTION,
      long_description_content_type=LONG_DESC_TYPE,
      author=AUTHOR,
      license=LICENSE,
      author_email=AUTHOR_EMAIL,
      url=URL,
      install_requires=INSTALL_REQUIRES,
      packages=find_packages()
      )